@@ -78,7 +78,6 @@ struct _GdmSessionDirectPrivate
 
         GdmSessionWorkerJob *job;
         GPid                 session_pid;
-        guint32              is_authenticated : 1;
         guint32              is_running : 1;
 
         /* object lifetime scope */
@@ -158,7 +157,8 @@ send_dbus_string_signal (GdmSessionDirect *session,
         dbus_message_iter_append_basic (&iter, DBUS_TYPE_STRING, &text);
 
         if (! send_dbus_message (session->priv->worker_connection, message)) {
-                g_debug ("GdmSessionDirect: Could not send %s signal", name);
+                g_debug ("GdmSessionDirect: Could not send %s signal",
+                         name ? name : "(null)");
         }
 
         dbus_message_unref (message);
@@ -334,7 +334,6 @@ gdm_session_direct_handle_authenticated (GdmSessionDirect *session,
         dbus_connection_send (connection, reply, NULL);
         dbus_message_unref (reply);
 
-        session->priv->is_authenticated = TRUE;
         _gdm_session_authenticated (GDM_SESSION (session));
 
         return DBUS_HANDLER_RESULT_HANDLED;
@@ -362,7 +361,6 @@ gdm_session_direct_handle_authentication_failed (GdmSessionDirect *session,
 
         g_debug ("GdmSessionDirect: Emitting 'authentication-failed' signal");
 
-        session->priv->is_authenticated = FALSE;
         _gdm_session_authentication_failed (GDM_SESSION (session), text);
 
         return DBUS_HANDLER_RESULT_HANDLED;
@@ -1558,9 +1556,7 @@ worker_exited (GdmSessionWorkerJob *job,
 {
         g_debug ("GdmSessionDirect: Worker job exited: %d", code);
 
-        if (!session->priv->is_authenticated) {
-                _gdm_session_authentication_failed (GDM_SESSION (session), NULL);
-        } else if (session->priv->is_running) {
+        if (session->priv->is_running) {
                 _gdm_session_session_exited (GDM_SESSION (session), code);
         }
 }
@@ -1572,9 +1568,7 @@ worker_died (GdmSessionWorkerJob *job,
 {
         g_debug ("GdmSessionDirect: Worker job died: %d", signum);
 
-        if (!session->priv->is_authenticated) {
-                _gdm_session_authentication_failed (GDM_SESSION (session), NULL);
-        } else if (session->priv->is_running) {
+        if (session->priv->is_running) {
                 _gdm_session_session_died (GDM_SESSION (session), signum);
         }
 }
@@ -2050,7 +2044,6 @@ gdm_session_direct_close (GdmSession *session)
         g_hash_table_remove_all (impl->priv->environment);
 
         impl->priv->session_pid = -1;
-        impl->priv->is_authenticated = FALSE;
         impl->priv->is_running = FALSE;
 }
 
