@@ -123,11 +123,11 @@ gdm_parse_language_name (const char *name,
         error = NULL;
         re = g_regex_new ("^(?P<language>[^_.@[:space:]]+)"
                           "(_(?P<territory>[[:upper:]]+))?"
-                          "(.(?P<codeset>[-_0-9a-zA-Z]+))?"
+                          "(\\.(?P<codeset>[-_0-9a-zA-Z]+))?"
                           "(@(?P<modifier>[[:ascii:]]+))?$",
                           0, 0, &error);
         if (re == NULL) {
-                g_critical (error->message);
+                g_critical ("%s", error->message);
                 return;
         }
 
@@ -216,6 +216,24 @@ construct_language_name (const char *language,
         return name;
 }
 
+static void
+make_codeset_canonical_for_locale (const char  *name,
+                                   char       **codeset)
+{
+        char *old_locale;
+
+        old_locale = setlocale (LC_CTYPE, NULL);
+
+        if (setlocale (LC_CTYPE, name) == NULL) {
+                return;
+        }
+
+        g_free (*codeset);
+        *codeset = g_strdup (nl_langinfo (CODESET));
+
+        setlocale (LC_CTYPE, old_locale);
+}
+
 char *
 gdm_normalize_language_name (const char *name)
 {
@@ -234,6 +252,10 @@ gdm_normalize_language_name (const char *name)
                                  &territory_code,
                                  &codeset, &modifier);
 
+        if (codeset != NULL) {
+                make_codeset_canonical_for_locale (name, &codeset);
+        }
+
         normalized_name = construct_language_name (language_code,
                                                    territory_code,
                                                    codeset, modifier);
@@ -459,7 +481,7 @@ select_dirs (const struct dirent *dirent)
                                 struct stat st;
                                 char       *path;
 
-                                path = g_build_filename (GNOMELOCALEDIR, dirent->d_name, NULL);
+                                path = g_build_filename (LIBLOCALEDIR, dirent->d_name, NULL);
                                 if (g_stat (path, &st) == 0) {
                                         mode = st.st_mode;
                                 }
@@ -479,7 +501,7 @@ collect_locales_from_directory (void)
         int             ndirents;
         int             cnt;
 
-        ndirents = scandir (GNOMELOCALEDIR, &dirents, select_dirs, alphasort);
+        ndirents = scandir (LIBLOCALEDIR, &dirents, select_dirs, alphasort);
 
         for (cnt = 0; cnt < ndirents; ++cnt) {
                 add_locale (dirents[cnt]->d_name);
@@ -735,6 +757,7 @@ territories_parse_start_tag (GMarkupParseContext      *ctx,
         const char *acode_2;
         const char *acode_3;
         const char *ncode;
+        const char *territory_common_name;
         const char *territory_name;
 
         if (! g_str_equal (element_name, "iso_3166_entry") || attr_names == NULL || attr_values == NULL) {
@@ -744,6 +767,7 @@ territories_parse_start_tag (GMarkupParseContext      *ctx,
         acode_2 = NULL;
         acode_3 = NULL;
         ncode = NULL;
+        territory_common_name = NULL;
         territory_name = NULL;
 
         while (*attr_names && *attr_values) {
@@ -771,6 +795,11 @@ territories_parse_start_tag (GMarkupParseContext      *ctx,
                                 }
                                 ncode = *attr_values;
                         }
+                } else if (g_str_equal (*attr_names, "common_name")) {
+                        /* skip if empty */
+                        if (**attr_values) {
+                                territory_common_name = *attr_values;
+                        }
                 } else if (g_str_equal (*attr_names, "name")) {
                         territory_name = *attr_values;
                 }
@@ -779,6 +808,10 @@ territories_parse_start_tag (GMarkupParseContext      *ctx,
                 ++attr_values;
         }
 
+        if (territory_common_name != NULL) {
+                territory_name = territory_common_name;
+        }
+
         if (territory_name == NULL) {
                 return;
         }
